@@ -324,7 +324,8 @@
    ((char=? ch #\')
     (let* ((c1 (read-char))
 	   (sc (if (eqv? c1 #\\) (read-esc-char) (string c1))))
-      (if (not (char=? #\' (read-char))) (error "bad char lit"))
+      (if (not (char=? #\' (read-char)))
+	  (throw 'nyacc-error "read-c-chlit: bad char literal"))
       (cons '$chlit sc)))
    ((char-set-contains? c:cx ch)
     (let ((c1 (read-char)))
